function [F, errv] = ISPM_cor(m, n, desired_coh, alpha, Nit)

% Incoherence via Shifted Power Method
% Corrected version of the algorithm appeared in IEEE SPL, vol.24, Sept.2017
% "Designing Incoherent Frames With Only MatrixVector Multiplications"
%
% Input:
%   m,n         - frame size
%   desired_coh - desired coherence, used as target (if too small, the
%                 algorithm does not converge)
%   alpha       - trade-off parameter; should be 0.5 or larger
%   Nit         - number of iterations (the algorithm does not stop bases
%                 on an error criterion
% Output:
%   F           - the frame
%   errv        - vector of mutual coherences at each iteration

% Note: the correction regards eq.(6): a square root should be applied on
% the right hand side

% BD 3.05.2018

% start with UNTF
F = gen_rand_untf(m,n);

errv = zeros(1,Nit);
err_min = 1;

% main loop
for k = 1 : Nit
  perm = randperm(n-1)+1;
  for j = perm  % a round of the atoms in random order
    d = F(:,j);
    v = F'*d;
    v(j) = 0;
    w = max(abs(v)/desired_coh, 1);
    w_max = max(w);
    eig_shift = (n/m)*alpha*w_max + (1-alpha)*sum(w)/m; % compute shift
    d = F*(w.*v) - eig_shift*d; % power method iteration
    d = d / norm(d);
    F(:,j) = d;
  end
  errv(k) = max(max(abs(F'*F - eye(n))));
  if errv(k) < err_min
    err_min = errv(k);
    F_best = F;
  end
end

% make sure that it is UNTF (most likely useless)
F = F_best;
[U,~,V] = svd(F);
F = U*V(:,1:m)'*sqrt(n/m);
